import os
import yaml
import boto3
import logging
import threading


from .task import Task
from .utils.template import Template
from .utils.tags import Tags
from .utils.loader import Loader

level = os.environ.get("EC2_DEMO_LOG", "info").upper()
logging.basicConfig(level=level)

class Instances:

    def __init__(self, env):
        self.client = boto3.client("ec2")
        self.env = env
        self.loadEnv()


    def loadEnv(self):
        with open(f"config/{self.env}.yaml", "r") as f:
            self.vars = yaml.safe_load(f)
        logging.debug(f"self.vars: {self.vars}")


    def create(self, path: str) ->list[str]:
        ids = []
        threads = []
        tasks = {}
        logging.debug(f"path: {path}")
        rendered = Template.render(path, self.vars)
        data = Loader.load(rendered)

        for name, config in data["instances"].items():
            task = Task(self.vars)
            thread = threading.Thread(
                name=f"Task {name}",
                target=task.create,
                args=[name, config],
            )
            tasks[name] = task
            threads.append(thread)
            thread.start()

        for thread in threads:
            thread.join()

        for name, task in tasks.items():
            for instanceId in task.instanceIds:
                ids.append(instanceId)
            # instaceIds = self.createInstance(name, config)
            # for instaceId in instaceIds:
            #     ids.append(instaceId)

        return ids


    def createInstance(self, name: str, config: dict) ->list[str]:
        ids = []
        envTags = self.vars["tags"]
        envTags.update({"Name": name})
        awsTags = Tags.build(envTags)

        logging.debug(f"Name: {name}")
        logging.debug(f"Config: {config}")
        logging.debug(f"Spec: {config['spec']}")
        spec = config["spec"]
        spec["TagSpecifications"] = [
            {
                "ResourceType": "instance",
                "Tags": awsTags,
            },
        ]
        spec["MinCount"] = 1
        spec["MaxCount"] = 1

        logging.debug(f"Patched spec: {spec}")
        res = self.client.run_instances(**spec)

        for instance in res["Instances"]:
            logging.debug(f"InstanceId: {instance['InstanceId']}")
            ids.append(instance["InstanceId"])

        return ids


    def delete(self, ids: list[str]) -> dict:
        status = {}
        logging.debug(f"Deleting ids: {ids}")
        res = self.client.terminate_instances(
            InstanceIds=ids,
        )
        for instance in res["TerminatingInstances"]:
            status[instance["InstanceId"]] = instance["CurrentState"]["Name"]

        return status


    def list(self) -> list:
        # Boto3 resources returns the ip addresses
        self.ec2 = boto3.resource("ec2")
        resources = []
        logging.debug(f"Finding ec2-demo instances...")
        res = self.ec2.instances.filter(
            Filters=[
                {
                    "Name": "tag:CreatedBy",
                    "Values": [
                        "ec2-demo",
                    ],
                },
            ],
        )
        return res



import sys
import yaml
import click
import logging
import tabulate

from .instances import Instances
from .utils.tags import Tags


@click.group()
def ec2():
    """Manages EC2 instances"""


@ec2.command()
@click.argument("env", type=str, required=True)
@click.argument("path", type=str, required=True)
def create(env, path):
    logging.debug(f"env: {env}")
    logging.debug(f"path: {path}")
    instances = Instances(env)
    ids = instances.create(path)
    for id_ in ids:
        print(f"- Instance \033[32;1m{id_}\033[0m created.")


@ec2.command()
@click.argument("env", type=str, required=True)
def list(env):
    logging.debug(f"env: {env}")
    instances = Instances(env)
    resources = instances.list()
    table = []
    headers = ["INSTANCE ID", "NAME", "STATE", "PRIVATE IP", "PUBLIC IP"]
    for instance in resources:
        name = Tags.find(instance.tags, "Name")
        table.append([
            instance.instance_id,
            name,
            instance.state["Name"],
            instance.private_ip_address,
            instance.public_ip_address,
        ])

    print(tabulate.tabulate(table,headers, tablefmt="rounded_outline"))


@ec2.command()
@click.argument("env", type=str, required=True)
@click.argument("ids", type=str, required=False, nargs=-1)
@click.option("--all", is_flag=True, default=False, help="Deletes all instances")
def delete(env, ids, all):
    logging.debug(f"env: {env}")
    logging.debug(f"ids: {ids}")
    logging.debug(f"all: {all}")

    instances = Instances(env)

    if all:
        resources = instances.list()
        ids = []
        for resource in resources:
            ids.append(resource.instance_id)
        status = instances.delete(ids)

    else:
        status = instances.delete(ids)

    for id_, state in status.items():
        print(f"- Instance \033[32;1m{id_}\033[0m is now in state \033[32;1m{state}\033[0m.")


def main():
    ec2()


if __name__ == "__main__":
    main()



import boto3
import logging
import time

from .utils.tags import Tags


class Task:

    def __init__(self, vars):
        self.vars = vars
        self.client = boto3.client("ec2")
        self.instanceIds = []

    def create(self, name: str, config: dict) ->list[str]:
        envTags = self.vars["tags"]
        envTags.update({"Name": name})
        awsTags = Tags.build(envTags)

        logging.debug(f"Name: {name}")
        logging.debug(f"Config: {config}")
        logging.debug(f"Spec: {config['spec']}")
        spec = config["spec"]
        spec["TagSpecifications"] = [
            {
                "ResourceType": "instance",
                "Tags": awsTags,
            },
        ]
        spec["MinCount"] = 1
        spec["MaxCount"] = 1

        logging.debug(f"Patched spec: {spec}")
        res = self.client.run_instances(**spec)

        for instance in res["Instances"]:
            logging.debug(f"InstanceId: {instance['InstanceId']}")
            self.instanceIds.append(instance["InstanceId"])

        time.sleep(2)
        return self.instanceIds



from setuptools import setup
from ec2_demo import VERSION

with open("README.md", "r") as f:
  description = f.read()

setup(
  name="ec2-demo",
  version=VERSION,
  description="EC2 demo",
  long_description=description,
  long_description_content_type="text/markdown",
  author="jpedro",
  author_email="jpedro.barbosa@gmail.com",
  url="https://github.com/jpedro/ec2-demo",
  download_url="https://github.com/jpedro/ec2-demo/tarball/master",
  keywords="ec2 create delete demo",
  license="MIT",
  python_requires='>=3',
  classifiers=[
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "Topic :: Software Development :: Build Tools",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3",
  ],
  packages=[
    "ec2_demo",
    "tests",
  ],
  install_requires=[
    "pyyaml",
    "jinja2",
    "requests",
    "boto3",
    "tabulate",
  ],
  entry_points={
    "console_scripts": [
      "ec2-demo=ec2_demo.cli:main",
    ],
  },
)



from unittest import TestCase, mock

from ec2_demo.instances import Instances


class TestInstances(TestCase):

    @mock.patch("ec2_demo.instances.Instances.create")
    def test_create(self, mocked):
        expected = ["i-created"]
        mocked.return_value = expected
        instances = Instances("dev")
        returned = instances.create("instances/test.yaml")
        self.assertEqual(expected, returned)


    @mock.patch("ec2_demo.instances.Instances.delete")
    def test_delete(self, mocked):
        expected = ["i-deleted"]
        mocked.return_value = expected
        instances = Instances("dev")
        returned = instances.delete("nobody-reads-this")
        self.assertEqual(expected, returned)


if __name__ == "__main__":
    unittest.main()
